
#!/bin/bash

from_sec_to_formatted_time () {
    time_in_seconds=$1

    days=$(( $time_in_seconds / 86400 ))

    residue=$(( ${time_in_seconds} - ${days}*86400 ))

    hours=$(printf "%02d" $(( ${residue} / 3600 )))

    residue=$(( ${residue} - ${hours}*3600 ))

    minutes=$(printf "%02d" $(( ${residue} / 60 )))

    seconds=$(printf "%02d" $(( ${residue} - ${minutes}*60 )))

    out=""
    if ! [ ${days} -eq 0 ]
    then
        out="${days}-"
    fi
    out="${out}${hours}:${minutes}:${seconds}"

    echo ${out}
} 

from_formatted_time_to_sec () {
    formatted_time=$1
    time_in_seconds=0

    if [[ $formatted_time == *"-"* ]]
    then
        # Time format == D-HH:MM:SS
        days=${formatted_time%-*}

        hhmmss=${formatted_time#*-}

        IFS=$':'
        hhmmss=( ${hhmmss} )
        unset ${IFS}

        hh=${hhmmss[0]}
        mm=${hhmmss[1]}
        ss=${hhmmss[2]%.*}

        ## TODO: AGGIUSTARE i decimali

        time_in_seconds=$(( ${days}*86400 + ${hh}*3600 + ${mm}*60 + ${ss} ))
    else
        # Time format == HH:MM:SS or MM:SS
        IFS=$':'
        tmp=($formatted_time)
        unset ${IFS} ## TODO: Qui da errore... da capirne il motivo

        if [[ ${#tmp[@]} == 3 ]]
        then
            # Time format == HH:MM:SS
            hh=${tmp[0]}
            mm=${tmp[1]}
            ss=${tmp[2]%.*}

            ## TODO: AGGIUSTARE i decimali

            time_in_seconds=$(( ${hh}*3600 + ${mm}*60 + ${ss} ))
        else
            # Time format == MM:SS
            mm=${tmp[0]}
            ss=${tmp[1]%.*}

            ## TODO: AGGIUSTARE i decimali

            time_in_seconds=$(( ${mm}*60 + ${ss} ))
        fi
    fi

    echo ${time_in_seconds}
}

declare -A job
job["id"]=$1

IFS=$'\n'
job_info=$(sacct -P -n -a --format JobID,User,Account,State,AllocCPUS,REQMEM,TotalCPU,ElapsedRaw,MaxRSS,ExitCode,NodeList,AllocNodes,Elapsed -j ${job["id"]})
unset ${IFS}

username=""
account=""

job["TotalRSS"]=0

line_counter=0
for line in ${job_info[@]}
do
    # Get Step info
    IFS=$'|'
    step_info=(${line})
    unset IFS

    # Get common info
    if [ ${line_counter} -eq 0 ]
    then
        username=${step_info[1]}
        account=${step_info[2]}
        job["State"]=${step_info[3]}
        job["AllocCPUS"]=${step_info[4]}
        job["REQMEM"]=${step_info[5]}
        job["TotalCPU"]=${step_info[6]}
        job["ElapsedRaw"]=${step_info[7]}
        job["ExitCode"]=${step_info[9]}
        job["NodeList"]=${step_info[10]}
        job["AllocNodes"]=${step_info[11]}
        job["Elapsed"]=${step_info[12]}
        job["CPUSperNode"]=$(( ${job["AllocCPUS"]} / ${job["AllocNodes"]} ))
        job["CoreWallTimeRaw"]=$(( ${job["ElapsedRaw"]} * ${job["AllocCPUS"]} ))
        job["CoreWallTime"]=$( from_sec_to_formatted_time ${job["CoreWallTimeRaw"]} ) 
        job["TotalCPURaw"]=$( from_formatted_time_to_sec ${job["TotalCPU"]} )
        job["CPUEfficiency"]=$( printf %.2f\\n "$((1000000 * ${job["TotalCPURaw"]} / ${job["CoreWallTimeRaw"]}))e-4" )
    else 

        # Get unit from TotalRSS
        unit=${step_info[8]: -1}
        # Raise an error if different from kiB
        if (( unit != "K" ))
        then
            echo "ERROR: unit of MaxRSS ($unit) unrecongnized"
            exit 1
        fi
        step_MAXRSS=${step_info[8]%K} 
        if (( step_MAXRSS > job["TotalRSS"] ))
        then
            job["TotalRSS"]=${step_MAXRSS}
        fi
    fi

    line_counter=$(( ${line_counter} + 1 ))
done

# Convertion for kiB to GiB
job["TotalRSS_GiB"]=$( printf %.2f\\n "$((10000 * ${job["TotalRSS"]} / 1048576))e-4" )

# Get unit from REQMEM
unit=${job["REQMEM"]: -1}
# Convert REQMEM to kiB
if [[ $unit == "G" ]]
then
    # from GiB to kiB (1024**2==1048576)
    job["REQMEM_kiB"]=$(( ${job["REQMEM"]%G}*1048576 ))
    job["REQMEM_GiB"]=${job["REQMEM"]%G}
else
    if [[ $unit == "M" ]]
    then
        # from GiB to kiB (1024**2==1048576)
        job["REQMEM_kiB"]=$(( ${job["REQMEM"]%M}*1024 ))
        job["REQMEM_GiB"]=$( printf %.2f\\n "$((10000 * ${job["REQMEM"]%M} / 1024))e-4" )
    else
        echo "ERROR: unit of REQMEM ($unit) unrecongnized"
        exit 2
    fi
fi

job["MEMEfficiency"]=$( printf %.2f\\n "$((1000000 * ${job["TotalRSS"]} / ${job["REQMEM_kiB"]}))e-4" )

#for key in ${!job[@]}
#do
#    echo "${key} -> ${job[$key]}"
#done

echo "Job ID:  ${job["id"]}"
echo "User:    ${username}"
echo "Account: ${account}"
echo "State: ${job["State"]} (exit code ${job["ExitCode"]%:*})"
echo "Nodes: ${job["AllocNodes"]}"
echo "Cores per node: ${job["CPUSperNode"]}"
echo "CPU Utilized: ${job["TotalCPU"]}"
echo "CPU Efficiency: ${job["CPUEfficiency"]}% of ${job["CoreWallTime"]} core-walltime"
echo "Job Wall-clock time: ${job["Elapsed"]}"
echo "Memory Utilized: ${job["TotalRSS_GiB"]} GiB"
echo "Memory Efficiency: ${job["MEMEfficiency"]}% of ${job["REQMEM_GiB"]} GiB"
